-- #####################################################################
--  KAP4: Demo 1 - Kreuztabelle klassisch
-- #####################################################################

SELECT SUM(CASE WHEN MONAT=201301 THEN UMSATZ ELSE 0 END) AS "201301",
              SUM(CASE WHEN MONAT=201302 THEN UMSATZ ELSE 0 END) AS "201302",
              SUM(CASE WHEN MONAT=201303 THEN UMSATZ ELSE 0 END) AS "201303"
FROM (
         SELECT convert(VARCHAR(6), BESTELL_DATUM,112) AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
         FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
         GROUP BY convert(VARCHAR(6), BESTELL_DATUM,112) 
      ) ums
WHERE MONAT IN(201301, 201302, 201303);

-- #####################################################################
--  KAP4: Demo 2 - Kreuztabelle mit PIVOT
-- #####################################################################

SELECT *
FROM ( 
         SELECT convert(VARCHAR(6), BESTELL_DATUM,112) AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
         FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
         GROUP BY convert(VARCHAR(6), BESTELL_DATUM,112)
     ) ums
PIVOT (SUM(UMSATZ) FOR MONAT IN([201301],[201302],[201303]) ) AS p;

-- #####################################################################
--  KAP4: Demo 3 - UNPIVOT
-- #####################################################################

SELECT *
FROM (
  SELECT *
  FROM (
         SELECT convert(VARCHAR(6), BESTELL_DATUM,112) AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
         FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
         GROUP BY convert(VARCHAR(6), BESTELL_DATUM,112) 
        ) ums
  PIVOT (SUM(UMSATZ) FOR MONAT IN([201301],[201302],[201303]) ) as p
) ums_piv
UNPIVOT (UMSATZ FOR MONAT IN("201301", "201302", "201303")  ) as up;

-- #####################################################################
--  KAP4: Demo 4 - COALESCE
-- #####################################################################

SELECT COALESCE(feb.PRODUKT_NR, mai.PRODUKT_NR) AS PRODUKT_NR,
              COALESCE(feb.BEZEICHNUNG, mai. BEZEICHNUNG) AS BEZEICHNUNG,
              feb.UMSATZ_FEB,
             mai.UMSATZ_MAI
FROM (
      SELECT p.PRODUKT_NR, p.BEZEICHNUNG,
                   SUM(bp.ANZAHL*p.PREIS) AS UMSATZ_FEB
      FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
      WHERE convert(VARCHAR(6), BESTELL_DATUM,112)='201302'
      AND b.VERKAEUFER_NR=6
      GROUP BY p.PRODUKT_NR, p.BEZEICHNUNG
          ) feb
FULL JOIN (
      SELECT p.PRODUKT_NR, p.BEZEICHNUNG,
                   SUM(bp.ANZAHL*p.PREIS) AS UMSATZ_MAI
      FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
      WHERE convert(VARCHAR(6), BESTELL_DATUM,112)='201305'
      AND b.VERKAEUFER_NR=6
      GROUP BY p.PRODUKT_NR, p.BEZEICHNUNG
     ) mai
ON feb.PRODUKT_NR=mai.PRODUKT_NR;


-- #####################################################################
--  KAP4: Demo 5 - WITH
-- #####################################################################

WITH  prod_ums AS (
            SELECT p.PRODUKT_NR, p.BEZEICHNUNG,
                          SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
            FROM TBL_PRODUKT p
                JOIN TBL_BESTELLUNG_POS bp ON bp.PRODUKT_NR=p.PRODUKT_NR
            GROUP BY p.PRODUKT_NR, p.BEZEICHNUNG
           ),
           ums_avg  AS (
                   SELECT AVG(UMSATZ) AS AVG_UMSATZ
                   FROM prod_ums
          )
SELECT *
FROM prod_ums
WHERE UMSATZ>(SELECT AVG_UMSATZ FROM ums_avg);
