-- #####################################################################
--  KAP9: bung 1
-- #####################################################################

CREATE VIEW V_UMSATZ_PRODUKT AS
SELECT  p.PRODUKT_GRUPPE_NR,
        p.PRODUKT_NR,
        p.BEZEICHNUNG,
        p.PREIS,
        SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
FROM TBL_BESTELLUNG_POS bp 
  JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
GROUP BY p.PRODUKT_GRUPPE_NR,
        p.PRODUKT_NR,
        p.BEZEICHNUNG,
        p.PREIS
ORDER BY 1,2;

-- #####################################################################
--  KAP9: bung 2
-- #####################################################################

CREATE VIEW V_SEC_KUNDE AS
SELECT KUNDE_NR, VORNAME, NACHNAME
FROM TBL_KUNDE;

GRANT SELECT ON V_SEC_KUNDE TO TEST3;


-- #####################################################################
--  KAP9: bung 3
-- #####################################################################

SELECT  KUNDE_NR,
        CASE WHEN ANZ_BESTELLUNGEN>5 THEN 'ja' ELSE 'nein' END AS MEHR_ALS_5_BEST
FROM (
    SELECT k.KUNDE_NR, Count(distinct BESTELLUNG_NR) AS ANZ_BESTELLUNGEN
    FROM TBL_KUNDE k
      JOIN TBL_BESTELLUNG b ON k.KUNDE_NR=b.KUNDE_NR
    GROUP BY k.KUNDE_NR
)
ORDER BY 1;

-- #####################################################################
--  KAP9: bung 4
-- #####################################################################

SELECT DISTINCT
    p.PRODUKT_NR,
    p.BEZEICHNUNG,
    SUM(bp.ANZAHL*p.PREIS) OVER (PARTITION BY p.PRODUKT_NR) AS UMSATZ,
    SUM(bp.ANZAHL*p.PREIS) OVER (PARTITION BY p.PRODUKT_GRUPPE_NR) AS UMSATZ_GRP
FROM TBL_BESTELLUNG_POS bp
  JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
ORDER BY 1 ASC

-- #####################################################################
--  KAP9: bung 35 (Seite 54)
-- #####################################################################

SELECT DISTINCT 
    x.*,
    SUM(UMSATZ) OVER(
      ORDER BY MONAT ASC
      ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING
    ) AS UMSATZ_KUM    
FROM (
    SELECT DISTINCT
        to_char(b.BESTELL_DATUM,'MM/YYYY') AS MONAT,
        SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY to_char(b.BESTELL_DATUM,'MM/YYYY')) AS UMSATZ
    FROM TBL_BESTELLUNG b
      JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
      JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
    WHERE to_char(b.BESTELL_DATUM,'YYYY')='2013'
) x
ORDER BY 1;


-- #####################################################################
--  KAP9: Aufgabe 1
-- #####################################################################

--> Anmelden als Benutzer TEST

REVOKE ALL PRIVILEGES ON TBL_KUNDE FROM PUBLIC;

GRANT UPDATE (PREIS) ON TBL_PRODUKT TO TEST2;

GRANT SELECT ON TBL_PRODUKT TO TEST2;

--> Anmelden als Benutzer TEST2 und Test der Rechte

Select *
from TEST.TBL_KUNDE; --> Sollte nicht mglich sein.


UPDATE TEST.TBL_PRODUKT
SET PREIS=1.0
WHERE PRODUKT_NR=16;

COMMIT;

Select *
from TEST.TBL_PRODUKT;

-- #####################################################################
--  KAP9: Aufgabe 2
-- #####################################################################

CREATE VIEW V_UMSATZ_GESAMT AS
SELECT SUM(UMSATZ) AS UMSATZ_GESAMT
FROM V_UMSATZ_PRODUKT;

ALTER TABLE TBL_PRODUKT ADD(UMSATZ_PRODUKT NUMBER(10,2));

UPDATE TBL_PRODUKT p
SET UMSATZ_PRODUKT= (SELECT UMSATZ FROM V_UMSATZ_PRODUKT WHERE PRODUKT_NR=p.PRODUKT_NR);

commit;

ALTER TABLE TBL_PRODUKT ADD(UMSATZ_ANTEIL NUMBER(10,2));

UPDATE TBL_PRODUKT
SET UMSATZ_ANTEIL=round(UMSATZ_PRODUKT/(SELECT UMSATZ_GESAMT FROM V_UMSATZ_GESAMT),2);

commit;

-- #####################################################################
--  KAP9: Aufgabe 3
-- #####################################################################

SELECT DISTINCT
      x.PRODUKT_GRUPPE_NR,
      FIRST_VALUE(BEZEICHNUNG) OVER(PARTITION BY x.PRODUKT_GRUPPE_NR ORDER BY PRODUKT_GRUPPE_NR, UMS_PROD DESC) TOP_NACH_UMSATZ,
      FIRST_VALUE(BEZEICHNUNG) OVER(PARTITION BY x.PRODUKT_GRUPPE_NR ORDER BY PRODUKT_GRUPPE_NR, BEST_PROD DESC) TOP_NACH_BEST
FROM (
    SELECT DISTINCT
          grp.PRODUKT_GRUPPE_NR,
          p.PRODUKT_NR,
          p.BEZEICHNUNG,
          SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY p.PRODUKT_NR) UMS_PROD,
          COUNT(distinct b.BESTELLUNG_NR) OVER(PARTITION BY p.PRODUKT_NR) BEST_PROD
    FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
          JOIN TBL_PRODUKTGRUPPE grp ON p.PRODUKT_GRUPPE_NR=grp.PRODUKT_GRUPPE_NR
) x
ORDER BY 1;

-- #####################################################################
--  KAP9: Aufgabe 4
-- #####################################################################

SELECT x.*,
       DENSE_RANK() OVER(PARTITION BY GRP_NR ORDER BY UMS_PROD DESC) AS RANK_GRP,
       DENSE_RANK() OVER(ORDER BY UMS_PROD DESC) AS RANK_GES,
       CASE WHEN (DENSE_RANK() OVER(PARTITION BY GRP_NR ORDER BY UMS_PROD DESC))=1 THEN '*' ELSE ' ' END AS TOP
FROM (       
    SELECT  DISTINCT
            grp.PRODUKT_GRUPPE_NR AS GRP_NR,
            grp.BEZEICHNUNG AS GRP_BEZ,
            p.PRODUKT_NR,
            p.BEZEICHNUNG AS PRODUKT_BEZ,
            SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY p.PRODUKT_NR) AS UMS_PROD
    FROM TBL_BESTELLUNG_POS bp
      JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
      JOIN TBL_PRODUKTGRUPPE grp ON p.PRODUKT_GRUPPE_NR=grp.PRODUKT_GRUPPE_NR
) x
ORDER by 1,6;