-- #####################################################################
--  KAP2: Demo 1 - Berechnung Umsatzanteil klassisch
-- #####################################################################

  
SELECT  p.PRODUKT_NR,
        p.BEZEICHNUNG,
        SUM(bp.ANZAHL*p.PREIS) AS UMSATZ_PROD,
        ges.UMSATZ_GES,
        round(SUM(bp.ANZAHL*p.PREIS)/ges.UMSATZ_GES*100,1) AS ANTEIL
FROM TBL_PRODUKT p
  JOIN TBL_BESTELLUNG_POS bp ON p.PRODUKT_NR=bp.PRODUKT_NR
  CROSS JOIN (
        SELECT SUM(bp.ANZAHL*p.PREIS) AS UMSATZ_GES
        FROM TBL_PRODUKT p
          JOIN TBL_BESTELLUNG_POS bp ON p.PRODUKT_NR=bp.PRODUKT_NR  
        ) ges 
GROUP BY p.PRODUKT_NR,
        p.BEZEICHNUNG,
        ges.UMSATZ_GES
ORDER BY ANTEIL DESC;

-- #####################################################################
--  KAP2: Demo 2 - Berechnung Umsatzanteil mit Analytischen Funktionen
-- #####################################################################
SELECT  DISTINCT
        p.PRODUKT_NR,
        p.BEZEICHNUNG,
        
        SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY p.PRODUKT_NR) AS UMSATZ_PRD,
        SUM(bp.ANZAHL*p.PREIS) OVER() AS UMSATZ_GES,
        
        round(SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY p.PRODUKT_NR)/
        SUM(bp.ANZAHL*p.PREIS) OVER()*100,1)  AS UMSATZ_ANTEIL
FROM TBL_PRODUKT p
  JOIN TBL_BESTELLUNG_POS bp ON p.PRODUKT_NR=bp.PRODUKT_NR
ORDER BY UMSATZ_ANTEIL DESC;

-- #####################################################################
--  KAP2: Demo 3 - Beispiel Row_Number()
-- #####################################################################

SELECT  DISTINCT
        p.PRODUKT_NR,
        p.BEZEICHNUNG,
        
        Row_Number() Over(ORDER BY PRODUKT_NR DESC) AS RowNumber
FROM TBL_PRODUKT p
ORDER BY PRODUKT_NR ASC;

-- #####################################################################
--  KAP2: Demo 4 - Berechnung kumulierter Umsatz
-- #####################################################################
    
SELECT  MONAT,
        UMSATZ,
        SUM(UMSATZ) OVER(PARTITION BY substr(MONAT,1,4) 
            ORDER BY MONAT 
            ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS UMSATZ_KUM
FROM (
        SELECT  to_char(b.BESTELL_DATUM,'YYYYMM') AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
        FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
        GROUP BY to_char(b.BESTELL_DATUM,'YYYYMM')
     ) ums
ORDER BY MONAT;

-- #####################################################################
--  KAP2: Demo 5 - ROWS Fenster mit Offset
-- #####################################################################
    
SELECT  MONAT,
        UMSATZ,
        SUM(UMSATZ) OVER(PARTITION BY substr(MONAT,1,4) 
            ORDER BY MONAT 
            ROWS BETWEEN 1 PRECEDING AND 1 PRECEDING) AS UMS_VORMONAT
FROM (
        SELECT  to_char(b.BESTELL_DATUM,'YYYYMM') AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
        FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
        GROUP BY to_char(b.BESTELL_DATUM,'YYYYMM')
     ) ums
ORDER BY MONAT;

    
-- #####################################################################
--  KAP2: Demo 6 - RANGE Fenster
-- #####################################################################
    
SELECT  MITARBEITER_NR, VORNAME, NACHNAME, GEHALT,
        SUM(GEHALT) OVER(ORDER BY GEHALT) AS GEH_RANGE,
        SUM(GEHALT) OVER(ORDER BY GEHALT ROWS BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS GEH_ROWS
FROM TBL_MITARBEITER;
    
    
-- #####################################################################
--  KAP2: Demo 7 - Kum. Umsatz mit RANGE
-- #####################################################################
    
SELECT DISTINCT 
        to_char(b.BESTELL_DATUM,'YYYYMM') AS MONAT,
        SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY to_char(b.BESTELL_DATUM,'YYYYMM')) AS UMSATZ,
        SUM(bp.ANZAHL*p.PREIS) OVER(PARTITION BY to_char(b.BESTELL_DATUM,'YYYY')
            ORDER BY to_char(b.BESTELL_DATUM,'YYYYMM')
            RANGE BETWEEN UNBOUNDED PRECEDING AND CURRENT ROW) AS UMSATZ_KUM
FROM TBL_BESTELLUNG b
  JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
  JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
ORDER BY MONAT;
    
-- #####################################################################
--  KAP2: Demo 8 - Beispiel LAG/LEAD
-- #####################################################################

SELECT  MONAT,
        LAG(UMSATZ,1,NULL) OVER(ORDER BY MONAT ASC) AS UMSATZ_PREV,
        UMSATZ AS UMSATZ_AKT,
        round(((UMSATZ / LAG(UMSATZ,1,NULL) OVER(ORDER BY MONAT ASC))-1)*100,1) AS ABW
FROM (
        SELECT  to_char(b.BESTELL_DATUM,'YYYYMM') AS MONAT,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ 
        FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
        GROUP BY to_char(b.BESTELL_DATUM,'YYYYMM')
    )
ORDER BY MONAT;

-- #####################################################################
--  KAP2: Demo 9 - Beispiel First_Value / Last_Value
-- #####################################################################

SELECT DISTINCT
        MONAT,
        FIRST_VALUE(UMSATZ) OVER(PARTITION BY MONAT ORDER BY BESTELL_DATUM ASC) AS UMS_FIRST_DAY
FROM (
      SELECT  b.BESTELL_DATUM,
              to_char(BESTELL_DATUM,'YYYYMM') AS MONAT,
              SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
      FROM TBL_BESTELLUNG b
        JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
        JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
      GROUP BY b.BESTELL_DATUM, to_char(BESTELL_DATUM,'YYYYMM')
    ) day
ORDER BY MONAT;

-- #####################################################################
--  KAP2: Demo 10 - Beispiel NTH_VALUE
-- #####################################################################

SELECT DISTINCT 
        prd.*,
        NTH_VALUE(UMSATZ,1) OVER(PARTITION BY prd.PRODUKT_GRUPPE ORDER BY prd.PRODUKT_GRUPPE, UMSATZ DESC ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS BEST_PROD,
        NTH_VALUE(UMSATZ,2) OVER(PARTITION BY prd.PRODUKT_GRUPPE ORDER BY prd.PRODUKT_GRUPPE, UMSATZ DESC ROWS BETWEEN UNBOUNDED PRECEDING AND UNBOUNDED FOLLOWING) AS BEST_2ND_PROD
FROM(
      SELECT  LPAD(p.PRODUKT_NR,2,'0')||' - '||p.BEZEICHNUNG AS PRODUKT,
              pg.PRODUKT_GRUPPE_NR||' - '||pg.BEZEICHNUNG AS PRODUKT_GRUPPE,
              SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
      FROM TBL_BESTELLUNG_POS bp 
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
          JOIN TBL_PRODUKTGRUPPE pg ON p.PRODUKT_GRUPPE_NR=pg.PRODUKT_GRUPPE_NR
      GROUP BY LPAD(p.PRODUKT_NR,2,'0')||' - '||p.BEZEICHNUNG,
               pg.PRODUKT_GRUPPE_NR||' - '||pg.BEZEICHNUNG
      ) prd
ORDER BY PRODUKT;
        
-- #####################################################################
--  KAP2: Demo 11 - Beispiel RANK / DENSE_RANK
-- #####################################################################

SELECT DISTINCT
        QUARTAL,
        MONAT,
        UMSATZ,
        RANK() OVER(PARTITION BY QUARTAL ORDER BY UMSATZ DESC) AS RANG
FROM (
        SELECT  to_char(b.BESTELL_DATUM,'YYYYMM') AS MONAT, 
                to_char(b.BESTELL_DATUM,'YYYYQ') AS QUARTAL,
                SUM(bp.ANZAHL*p.PREIS) AS UMSATZ 
        FROM TBL_BESTELLUNG b
          JOIN TBL_BESTELLUNG_POS bp ON b.BESTELLUNG_NR=bp.BESTELLUNG_NR
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
        GROUP BY to_char(b.BESTELL_DATUM,'YYYYMM'), to_char(b.BESTELL_DATUM,'YYYYQ')
      )
ORDER BY QUARTAL, RANG;

-- #####################################################################
--  KAP2: Demo 12 - Beispiel RATIO_TO_REPORT (nur ORA)
-- #####################################################################

SELECT  prd.*,
        round((RATIO_TO_REPORT(prd.UMSATZ) OVER())*100,1) AS ANTEIL
FROM (
      SELECT p.PRODUKT_NR,
             p.BEZEICHNUNG,
             SUM(bp.ANZAHL*p.PREIS) AS UMSATZ
      FROM TBL_BESTELLUNG_POS bp 
          JOIN TBL_PRODUKT p ON bp.PRODUKT_NR=p.PRODUKT_NR
      GROUP BY p.PRODUKT_NR, p.BEZEICHNUNG
      ) prd
ORDER BY ANTEIL DESC;    

